--[[
	Author:  Azmaedus
	Version: 0.4b
	compatible with game version 6.0.3
	
	Change Log
	----------
	11/29/2014 - 0.1 	Creation of addon
	11/29/2014 - 0.1.1 	Support for tier 1, 2, and 3 garrison (Outpost, Fort, and Garrison)
	12/01/2014 - r8		1st commit to wowace.com (alpha)
	12/02/2014 - r9		Added multi-dimensional table to hold music filename references
	12/26/2014 - r9		Fixed issue with music not playing inside garrison buildings by changing the way to detect if we are inside Lunarfall
	12/27/2014 - r10	2 minute timer added between random playlist items.
						Dropdown music now functional per expansion selected.
						Fixed reload (musiclist) not loading in profile; forgot DB at the end of savedvars .toc
	12/28/2014 - r11	Fixed /reload issue not playing music
	12/29/2014 - 0.2b	1st beta: Fixed error when opening the options UI
	12/29/2014 - 0.3b	Changed from using 2 table design to a single table for holding music information
	12/29/2014 - 0.3.1b	Added Evil Forest 1, 2 and 3 music (Silverpine & Tirisfal)
	12/30/2014 - 0.4b	Code review & optimization (1st pass)
						Re-design of options UI
						Added Audio tab to let users play a wow embedded file by copy/paste into the input control
						Added About tab with relative information
						Localization of music titles and options table.

	Known Bugs
	----------
	- typing /gjb for the first time during a character login session or after /reload does not bring up the addon options UI
	
]]

-- Load the Ace libraries --
GarrisonJukeBox = LibStub("AceAddon-3.0"):NewAddon("GarrisonJukeBox", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GarrisonJukeBox")

-- globals
local gVersion = "0.4b"
local gReleaseDate = "12/30/2014"
local gMusicPlaying = false 	-- is the music currently playing
local gMusicIndex = 1			-- current item index of table musicfiles
local gMaxMusic = 0				-- number of items in musicfiles table
local gOnLoadMusicTimer = 5		-- HACK!!! wait n seconds when PLAYYER_LOGIN is triggered before playing music (make sure Blizz level loading script doesn't override PlayMusic() call)
local gPlayerLogin = false		-- used to acknowledge that a player has logged in or reloaded ui
local LUNARFALL = 971			-- zone id of Lunarfall

local gSG_COLOR_GREEN = "|cff38ce39"
local gSG_COLOR_BLUE = "|cff539ce8"

-- temp storage for select vars
local expac = "i1wow"
local music = 1
local jukebox = ""
local playsoundfile = ""

-- Create the default settings for this addon --
local defaults = { 
    profile =  {
		musiclist = {},
		shuffle = true,
	},
}

--[[
	Author:  Azmaedus US-Arathor
	For use with GarrisonJukeBox
]]

-- table holding music info (name, file, length)
local musictable = {
	-- Classic
	i1wow = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\GlueScreenMusic\\wow_main_theme.mp3", 161},
		{L["TITLE_EVILFOREST_NIGHT"], "Sound\\Music\\ZoneMusic\\EvilForest\\NightEvilForest02.mp3", 75},
		{L["TITLE_MOLTENCORE"], "Sound\\Music\\Musical Moments\\battle\\battle01.mp3", 48},
		{L["TITLE_ENCHANTEDFOREST"], "Sound\\Music\\ZoneMusic\\EnchantedForest\\EnchantedForest01.mp3", 50},
	},

	-- The Burning Crusade
	i2bc = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\GlueScreenMusic\\BC_main_theme.mp3", 226},
		{L["TITLE_SUNWELL_FLYBY"], "Sound\\Music\\ZoneMusic\\Sunwell\\SW_SunwellFlybyUni01.mp3", 57},
		{L["TITLE_ESCDURNHOLDE"], "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_EscapeDurnIntroUni.mp3", 112},
		{L["TITLE_HYJAL"], "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_HyjalExtWalk8Uni.mp3", 70},
	},
	
	-- Wrath of the Lich King
	i3wotlk = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\GlueScreenMusic\\WotLK_main_title.mp3", 544},
	},
	
	-- Cataclysm
	i4cata = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\cataclysm\\MUS_Shattering_UU01.mp3", 725},
		{L["TITLE_NIGHTELVES"], "Sound\\Music\\cataclysm\\MUS_NightElves_GU01.mp3", 160},
	},
	
	-- Mists of Pandaria
	i5mop = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\Pandaria\\MUS_50_HeartofPandaria_01.mp3", 480},
		{L["TITLE_WANDERINGISLE"], "Sound\\Music\\ZoneMusic\\WanderingIsle\\MUS_50_WanderingIsleB_Hero_UU01.mp3", 135},
		{L["TITLE_VALLEY4WINDS"], "Sound\\Music\\Pandaria\\MUS_50_BalloonRide_Hero_01.mp3", 165},
		{L["TITLE_SONG_LIULANG"], "Sound\\Music\\Pandaria\\MUS_50_SongofLiulang_VOX_Hero_01.mp3", 170},
		{L["TITLE_SERPENTRACE"], "Sound\\Music\\Pandaria\\MUS_50_SerpentRiders_SerpentRace.mp3", 238},
		{L["TITLE_GINSUVALOR"], "Sound\\Music\\Pandaria\\MUS_50_ShadoPan_A_Hero_03.mp3", 172},
		{L["TITLE_WAYOTMONK"], "Sound\\Music\\Pandaria\\MUS_50_Monk_Windwalker_A_01.mp3", 157},
		
	},
	
	-- Warlords of Draenor
	i6wod = {
		{L["TITLE_MAIN_THEME"], "Sound\\Music\\Draenor\\MUS_60_ASiegeofWorlds_MainTitle.mp3", 726},
	},
	
	-- Warcraft III
	i7events = {
		{L["TITLE_POWEROTHORDE"], "Sound\\Music\\ZoneMusic\\DMF_L70ETC01.mp3", 282},
		{L["TITLE_FIREFESTIVAL"], "Sound\\Music\\WorldEvents\\AllianceFirepole.mp3", 68},
		{L["TITLE_ARGENTTOURNY"], "Sound\\Music\\ZoneMusic\\ArgentTournament\\AT_JoustEvent.mp3", 123},
		
	},
}

-- Create the options variable to use in the config screen --
local options = {
	name = L["OPT_JUKEBOX_OPTIONS"],
	handler = GarrisonJukeBox,
	type = "group",
	childGroups = "tab",
	args = {
		grp1jukebox = {
			name = L["OPT_JUKEBOX"],
			desc = L["OPT_JUKEBOX_DESC"],
			type = "group",
			order = 1,
			args = {
				hdrselection = {
					name = L["HDR_SELECTION"],
					type = "header",
					order = 1,
				},
				expac = {
					type = "select",
					name = L["SEL_EXPANSION"],
					desc = L["SEL_EXPANSION_DESC"],
					style = "dropdown",
					values = "GetExpanssions",
					set = function(info, value)
						expac = value
					end,
					get = "GetExpac",
					width = "normal",
					order = 2,
				},
				music = {
					type = "select",
					name = L["SEL_MUSIC"],
					desc = L["SEL_MUSIC_DESC"],
					style = "dropdown",
					values = "GetCurExpacMusicList",
					set = function(info, value)
						music = value
					end,
					get = "GetMusic",
					width = "normal",
					order = 5,
				},
				musicadd = {
					type = "execute",
					name = L["CMD_ADD"],
					desc = L["CMD_ADD_DESC"],
					func = function()
						local xps = GarrisonJukeBox:GetExpanssions()
						local xp = xps[GarrisonJukeBox:GetExpac()]
						local mus = GarrisonJukeBox:GetCurExpacMusicList()
						local mu = mus[GarrisonJukeBox:GetMusic()]
						
						-- TODO: Will we allow duplicates?
						table.insert(GarrisonJukeBox.db.profile.musiclist, xp .. "/" .. mu)
						jukebox = xp .. "/" .. mu
					end,
					width = "half",
					order = 10,
				},
				cmdplaysamplemusic = {
					type = "execute",
					name = L["OPT_PLAYSAMPLE"],
					desc = L["OPT_PLAYSAMPLE_DESC"],
					func = function()
						local xps = GarrisonJukeBox:GetExpanssions()
						local xp = xps[GarrisonJukeBox:GetExpac()]
						local mus = GarrisonJukeBox:GetCurExpacMusicList()
						local mu = mus[GarrisonJukeBox:GetMusic()]

						StopMusic()
						gMusicPlaying = false
						GarrisonJukeBox:CancelTimer(GarrisonJukeBox.musicTimer)
						PlayMusic(musictable[expac][music][2])
						GarrisonJukeBox:Print(L["NOW_PLAYING"] .. xp .. "/" .. mu)
					end,
					width = "normal",
					order = 15,
				},
				cmdstopsound = {
					type = "execute",
					name = L["CMD_STOP"],
					func = function()
						StopMusic()
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 16,
				},
				descspace1 = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 20,
				},
				hdrjukebox = {
					name = L["SEL_JUKEBOX"],
					type = "header",
					order = 25,
				},
				musiclist = {
					type = "select",
					name = L["SEL_JUKEBOX"],
					desc = L["SEL_JUKEBOX_DESC"],
					style = "dropdown",
					values = function()
						return GarrisonJukeBox.db.profile.musiclist
					end,
					set = function(info, value)
						jukebox = value
					end,
					get = function(info)
						return jukebox
					end,
					width = "double",
					order = 30,
				},
				musicdelete = {
					type = "execute",
					name = L["CMD_REMOVE"],
					desc = L["CMD_REMOVE_DESC"],
					func = function()
						GarrisonJukeBox.db.profile.musiclist[jukebox] = nil
					end,
					width = "half",
					order = 35,
				},
				musicdeleteall = {
					type = "execute",
					name = L["CMD_REMOVEALL"],
					desc = L["CMD_REMOVEALL_DESC"],
					func = function()
						GarrisonJukeBox.db.profile.musiclist = {}
						jukebox = ""
					end,
					width = "normal",
					order = 40,
				},
				descspaceshuffle = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 45,
				},
				shuffle = {
					type = "toggle",
					name = L["TOG_SHUFFLE"],
					desc = L["TOG_SHUFFLE_DESC"],

					set = function(info, value)
						GarrisonJukeBox.db.profile.shuffle = value
					end,

					get = function(info)
						return GarrisonJukeBox.db.profile.shuffle
					end,

					width = "full",
					order = 50,
				},
			},
		},
		grp2audio = {
			name = L["OPT_AUDIO"],
			desc = L["OPT_AUDIO_DESC"],
			type = "group",
			order = 10,
			--disabled = true,
			args = {
				txtplaysound = {
					name = L["INP_WOWAUDIOFILE"],
					type = "input",
					width = "double",
					set = function(info, value)
						playsoundfile = value
					end,

					get = function(info)
						return playsoundfile
					end,
					order = 10,
				},
				cmdplaysound = {
					type = "execute",
					name = L["CMD_PLAY"],
					func = function()
						StopMusic()
						gMusicPlaying = false
						GarrisonJukeBox:CancelTimer(GarrisonJukeBox.musicTimer)
						PlayMusic(playsoundfile)
						GarrisonJukeBox:Print(L["NOW_PLAYING"] .. playsoundfile)
					end,
					width = "half",
					order = 11,
				},
				cmdstopsound = {
					type = "execute",
					name = L["CMD_STOP"],
					func = function()
						StopMusic()
						GarrisonJukeBox:ProcessInfo()
					end,
					width = "half",
					order = 12,
				},
			},
		},
		grp3about = {
			name = L["OPT_ABOUT"],
			desc = "",
			type = "group",
			order = 20,
			args = {
				desctitle = {
					name = gSG_COLOR_GREEN .. L["ADDON_NAME"] .. " " .. gVersion .. "|r",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 1,
				},
				desctitle2 = {
					name = gSG_COLOR_GREEN .. L["ADDON_RELEASE"] .. gReleaseDate .. "|r",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 2,
				},
				descauthor = {
					name = gSG_COLOR_BLUE .. L["ADDON_AUTHOR"] .. " " .. "Azmaedus"	.. "|r",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 10,
				},
				descspace1 = {
					name = " ",
					type = "description",
					width = "full",
					fontSize = "large",
					order = 20,
				},
--[[				deschowto = {
					name = "Howto",
					type = "description",
					width = "full",
					fontSize = "medium",
					order = 30,
				},
				desccont1 = {
					name = "Jukebox tab",
					type = "description",
					width = "full",
					fontSize = "small",
					order = 40,
				},]]
			},
		},
	},
}

-- function to get a table of all expansion/category names with keys that match the options table
function GarrisonJukeBox:GetExpanssions()
	return {
		i1wow = EXPANSION_NAME0,
		i2bc = EXPANSION_NAME1,
		i3wotlk = EXPANSION_NAME2,
		i4cata = EXPANSION_NAME3,
		i5mop = EXPANSION_NAME4,
		i6wod = EXPANSION_NAME5,
		i7events = L["EXP_WCEVENTS"],
	}
end

-- options table functions that may be called from more than one component
function GarrisonJukeBox:GetCurExpacMusicList()
	local tbl = musictable[self:GetExpac()]
	local t = {}

	for k, v in ipairs(tbl) do
		table.insert(t, k, v[1])
	end
	
	return t
end

function GarrisonJukeBox:GetExpac(info)
	return expac
end

function GarrisonJukeBox:GetMusic(info)
	return music
end

-- OnInitialize
function GarrisonJukeBox:OnInitialize()
	GarrisonJukeBox.db = LibStub("AceDB-3.0"):New("GarrisonJukeBoxDB", defaults, true)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GarrisonJukeBoxDB", options)
	GarrisonJukeBox.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GarrisonJukeBoxDB", "Garrison JukeBox")

	self:RegisterChatCommand("gjb", "ChatCommand")

	gMusicIndex = 1
	gMusicPlaying = false
	gMaxMusic = 0
	gLoaded = false
	self.musicTimer = nil
end

-- OnLoad
function GarrisonJukeBox:OnLoad()
	GarrisonJukeBox:RegisterEvent("PLAYER_LOGIN", "OnLoginEvent")
	GarrisonJukeBox:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPEWEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED", "OnZoneEvent")
		
	self:Print(L["ADDON_NAME"] .. " " .. gVersion .. " " .. L["ADDON_LOADED"])
end

-- Events
function GarrisonJukeBox:OnLoginEvent(event, ...)
	StopMusic()
	gMusicPlaying = false
	gPlayerLogin = true
	self.musicTimer = self:ScheduleTimer("TimerEvent", gOnLoadMusicTimer)
end

function GarrisonJukeBox:OnPEWEvent(event, ...)
	if gPlayerLogin == false then
		StopMusic()
		gMusicPlaying = false
		self.musicTimer = self:ScheduleTimer("TimerEvent", gOnLoadMusicTimer)
	end
end

function GarrisonJukeBox:OnZoneEvent(event, ...)
	self:ProcessInfo()
end

-- Ace3 Timer event for playing music
function GarrisonJukeBox:TimerEvent()
	self:ProcessInfo()
	gPlayerLogin = false
end

-- function called to determine if music needs to be played after a registered event has fired
function GarrisonJukeBox:ProcessInfo()
	if GetCurrentMapAreaID() == LUNARFALL then
		gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
		if gMaxMusic > 0 then
			self:PlayMusicList()
		end
	else 
		if gMusicPlaying == true then
			StopMusic()
			gMusicPlaying = false
			self:CancelTimer(self.musicTimer)
		end
	end
end

-- Table Invert for query of key
function table_invert(t)
   local s={}
   for k,v in pairs(t) do
     s[v]=k
   end
   return s
end

-- break appart the musiclist item to find out what file to play in the musicfile table
-- returns a table representing the music info
function MusicInfoFromJukeBoxItem(s)
	local idx = 0
	
	-- get the expansion key
	local b = strfind(s, "/")
	local ns = strsub(s, 1, b-1)
	local xp = GarrisonJukeBox:GetExpanssions() -- 
	local xpi = table_invert(xp)
	idx = xpi[ns]

	-- get music info
	ns = strsub(s, b+1)
	local mt = musictable[idx] -- get table of music infos relative to the expansion
	local t = {}
	
	-- loop through tables to find the music info
	for k, v in ipairs(mt) do
		if v[1] == ns then
			t[1] = v[1]
			t[2] = v[2]
			t[3] = v[3]
			break
		end
	end
	
	return t
end

-- function to play an item in the musiclist table
function GarrisonJukeBox:PlayMusicList()
	gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
	if gMaxMusic > 0 then
		self:CancelTimer(self.musicTimer)
		
		-- check if the list has shrunk since last pass and index is greater than number of list items
		if gMusicIndex > gMaxMusic then
			gMusicIndex = gMaxMusic
		end
		
		-- check if the playlist is in shuffle mode
		if GarrisonJukeBox.db.profile.shuffle == true then
			gMusicIndex = math.random(1, gMaxMusic)
		end
		
		-- find what music to play
		local mli = GarrisonJukeBox.db.profile.musiclist[gMusicIndex]
		local info = MusicInfoFromJukeBoxItem(mli) -- gets a table from musictable[n]
		StopMusic()
		PlayMusic(info[2])
		self:Print(L["PLAYING"] .. "#" .. gMusicIndex .. " - " .. GarrisonJukeBox.db.profile.musiclist[gMusicIndex])
		gMusicPlaying = true
		self.musicTimer = self:ScheduleTimer("TimerEvent", info[3])

		-- increment index after item is playing
		if gMusicIndex == gMaxMusic then
			gMusicIndex = 1
		else
			gMusicIndex = gMusicIndex + 1
		end
	end
end

-- slashcmd
function GarrisonJukeBox:ChatCommand(input)
  -- Open dialog if no parameters, else do something with the parameters
  if not input or input:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory(GarrisonJukeBox.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(GarrisonJukeBox, "gjb", "GarrisonJukeBoxDB", input)
  end
end